#include <nksdynprogprob.h>

/*****************************************************************************
 
****************************** OVERALL ROUTINES ******************************
 
*****************************************************************************/

Nks_VecIndex::Nks_VecIndex (
void *initValue_,                    // initial value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for conversion (1 MemNew required) 
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
Nks_ValueIndexFct *val_,             // index -> value 
size_t dimProb_)                     // dim(prob) = max (# values) 
{
MSG("Entering Nks_VecIndex::Nks_VecIndex 7")
	dimProb = dimProb_;
   NKS_CATCH(prob = new double [dimProb_], std::bad_alloc, "Nks_VecIndex::Nks_VecIndex 7");
   prob = (double *) Nks_MemZero ((void *) prob, dimProb_ * sizeof (double)); 
	minProb = maxProb = (*ind_) (initValue_, indexParam_);
   NKS_CATCH(value = (void *) new char [maxSizeValue_], std::bad_alloc, "Nks_VecIndex::Nks_VecIndex 7");
	prob [minProb] = 1.0;
   sizeValue = sizeValue_;
	val = val_;
	ind = ind_;
	indexParam = indexParam_;
MSG("Leaving Nks_VecIndex::Nks_VecIndex 7")
}

Nks_VecIndex::Nks_VecIndex (
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t dimProb_)                     // dim(prob) = max (# values) 
{
MSG("Entering Nks_VecIndex::Nks_VecIndex 2")
	dimProb = dimProb_;
   NKS_CATCH(prob = new double [dimProb_], std::bad_alloc, "Nks_VecIndex::Nks_VecIndex 2");
	prob = (double *) Nks_MemZero ((void *) prob, dimProb_ * sizeof (double)); 
	minProb = maxProb = 0;
   NKS_CATCH(value = (void *) new char [maxSizeValue_], std::bad_alloc, "Nks_VecIndex::Nks_VecIndex 2");
MSG("Leaving Nks_VecIndex::Nks_VecIndex 2")
}

Nks_VecIndex::~Nks_VecIndex () 
{
MSG("Entering Nks_VecIndex::~Nks_VecIndex")
   delete [] value; 
   value = 0;
   delete [] prob; 
   prob = 0;
MSG("Leaving Nks_VecIndex::~Nks_VecIndex")
}

Nks_DynProgProb::Nks_DynProgProb (   // inputs initial dynamic prog value 
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
void *initValue_,                    // initial value 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for conversion 
size_t dimProb_)                     // dim(prob) = max (# values) 
{
MSG("Entering Nks_DynProgProb::Nks_DynProgProb")
   Initialize (*this, maxSizeValue_, sizeValue_, initValue_, val_, ind_, indexParam_, dimProb_);                   // parameters for conversion 
MSG("Leaving Nks_DynProgProb::Nks_DynProgProb")
}

void Nks_DynProgProb::Reinitialize ( // inputs initial dynamic prog value 
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
void *initValue_,                    // initial value 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for conversion 
size_t dimProb_)                     // dim(prob) = max (# values) 
{
MSG("Entering Nks_DynProgProb::Nks_DynProgProb")
   Destroy ();
   Initialize (*this, maxSizeValue_, sizeValue_, initValue_, val_, ind_, indexParam_, dimProb_);                   // parameters for conversion 
MSG("Leaving Nks_DynProgProb::Nks_DynProgProb")
}

Nks_DynProgProb::~Nks_DynProgProb ()  // parameters for conversion 
{
MSG("Entering Nks_DynProgProb::~Nks_DynProgProb")
   Destroy ();
MSG("Leaving Nks_DynProgProb::~Nks_DynProgProb")
}

void Nks_DynProgProb::DeleteCalculationSpace ()    // deallocate workspace 
{
MSG("Entering Nks_FreeNks_DynProgProb")
	delete index [current == 0 ? 1 : 0];
   index [current == 0 ? 1 : 0] = 0;
MSG("Leaving Nks_FreeNks_DynProgProb")
}

void Nks_DynProgProb::NewCalculationSpace ()    // reallocate workspace 
{
MSG("Entering Nks_FreeNks_DynProgProb")
   NKS_CATCH(index [current == 0 ? 1 : 0] = new Nks_VecIndex (maxSizeValue, index [current]->dimProb), 
      std::bad_alloc, "Nks_DynProgProb::NewCalculationSpace");
MSG("Leaving Nks_FreeNks_DynProgProb")
}

void Nks_DynProgProb::MoreCalculationSpace (     // extends dynamic prog values 
size_t maxSizeValue_,                // dim(prob) = new max (# values) 
size_t dimProb_)                     // expected maximum sizeof (value) 
{
MSG("Entering Nks_DynProgProb::MoreCalculationSpace")
   for (size_t i = 0; i < 2; i++) {
      if (dimProb_ == 0) {
		   delete [] index [i]->prob;
         index [i]->prob = 0;
	   } else {
		   NKS_CATCH(index [i]->prob = (double *) Nks_MemMore ((void *) index [i]->prob, \
			   dimProb_ * sizeof (double)), std::bad_alloc, "Nks_DynProgProb::MoreCalculationSpace");
	   }
	   index [i]->dimProb = dimProb_;
	   if (maxSizeValue_ == 0) {
		   delete index [i]->value;
		   index [i]->value = 0;
	   } else {
		   NKS_CATCH(index [i]->value = Nks_MemMore (index [i]->value, maxSizeValue_), \
            std::bad_alloc, "Nks_DynProgProb::MoreCalculationSpace");
	   }
   }
	maxSizeValue = maxSizeValue_;
MSG("Leaving Nks_DynProgProb::MoreCalculationSpace")
}

bool Nks_DynProgProb::Update (       // updates dynamic prog probs 
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for index function 
size_t dimStates_,                   // #(states) 
size_t sizeStates_,                  // memory size for states 
void *states_,                       // void *to beginning of states array 
bool isProb_,                        // ? is probStates a vector of prob's ? 
double *probStates_)                 // corresponding array of states prob's 
{
MSG("Entering Nks_DynProgProb::Update")
MSG("Leaving Nks_DynProgProb::Update")
return UpdateEither (updateFct_, updateParam_, sizeValue_, 
	val_, ind_, indexParam_, dimStates_, sizeStates_, states_, isProb_, 
	probStates_, 0, 0);
}

bool Nks_DynProgProb::UpdateConditional (         // updates dynamic prog probs cond'ally 
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for index function 
size_t dimStates_,                   // #(states) 
size_t sizeStates_,                  // memory size for states 
void *states_,                       // void *to beginning of states array 
bool isProb_,                        // ? does condProbFct produce prob's ? 
Nks_IndexCondProbFct *condProbFct_,  // func producing conditional prob's 
void *probParam_)                    // parameters for conditional prob's 
{
MSG("Entering Nks_DynProgProb::UpdateConditional")
MSG("Leaving Nks_DynProgProb::UpdateConditional")
return UpdateEither (updateFct_, updateParam_, sizeValue_, 
	val_, ind_, indexParam_, dimStates_, sizeStates_, states_, isProb_, 
	0, condProbFct_, probParam_);
}

bool Nks_DynProgProb::Traverse (     // outputs copies of dynamic prog probs 
Nks_BoolFct *boolfct_)               // BoolFct is applied to all values 
												 //    and stops when return is FALSE 
{
MSG("Entering Nks_DynProgProb::Traverse")
	for (long i = index [current]->minProb; i <= index [current]->maxProb; i++) {
		index [current]->val (index [current]->value, i, index [current]->indexParam);
		if (! (* boolfct_) (index [current]->ProbModIndex (i), index [current]->value)) {
return FALSE;
		}
	}
MSG("Leaving Nks_DynProgProb::Traverse")
return TRUE;
}

static Nks_VecIndex *setIndex;
static Nks_UpdateFct *setUpdateFct;

/*static*/ void Sss_ConvolveUpdate (void *newVecValue, void *oldVecValue, void *state, void *updateParam)
{
	(* setIndex->val) (setIndex->value, 
		(*(size_t *) (state)) + setIndex->minProb, setIndex->indexParam);
	(* setUpdateFct) (newVecValue, oldVecValue, setIndex->value, updateParam);
}

bool Nks_DynProgProb::ConvolveVecIntoVec (        // convolves vecFrom into this 
Nks_DynProgProb *dynProgProbFrom_,   // Nks_DynProgProb : pass as a void *
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index function for vector of values 
Nks_IndexValueFct *ind_,             // value function for indexes 
void *indexParam_)                   // parameters for index function 
{
MSG("Entering Nks_DynProgProb::ConvolveVecIntoVec")
	Nks_VecIndex *oldVec = setIndex = dynProgProbFrom_->index [dynProgProbFrom_->current];
	setUpdateFct = updateFct_;
MSG("Leaving Nks_DynProgProb::ConvolveVecIntoVec")
return Update (Sss_ConvolveUpdate, updateParam_, 
		sizeValue_, val_, ind_, indexParam_, oldVec->maxProb - oldVec->minProb + 1, 
		sizeof (size_t), 0, (bool) (this->isProb && dynProgProbFrom_->isProb), 
		oldVec->prob + oldVec->minProb);
}

/*****************************************************************************
 
******************************* UPDATE ROUTINE *******************************
 
*****************************************************************************/

bool Nks_DynProgProb::UpdateEither ( // updates dynamic prog probs 
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index function for vector of values 
Nks_IndexValueFct *ind_,             // value function for indexes 
void *indexParam_,                   // parameters for index function 
size_t dimStates_,                   // #(states) 
size_t sizeStates_,                  // memory size for states 
void *states_,                       // void *to beginning of states array 
bool isProb_,                        // ? is probStates a vector of prob's ? 
double *probStates_,                 // corresponding array of states prob's 
Nks_IndexCondProbFct *condProbFct_,  // func producing conditional prob's 
void *probParam_)                    // parameters for conditional prob's 
{
	size_t minProb = SSIZE_MAX;        
	size_t maxProb = 0;
	size_t i;
   Nks_VecIndex *oldVec = index [current];
   Nks_VecIndex *newVec = index [current == 0 ? 1 : 0];
	void *state = (void *) &i;
	long newIndex = 0;
	double probCond;
MSG("Entering Nks_DynProgProb::UpdateEither")
	if (probStates_ == 0 && 
		condProbFct_ == 0) {
Nks_Abort ("Nks_UpdateEither : probStates_ == 0 == condProbFct_");
	} else if (probStates_ != 0 && 
		condProbFct_ != 0) {
Nks_Abort ("Nks_UpdateEither : probStates_ != 0 != condProbFct_");
	} else if (maxSizeValue < sizeValue_) {
Nks_Abort ("Nks_UpdateEither : maxSizeValue < sizeValue_");
	}
	(void) Nks_MemZero ((void *) newVec->prob, newVec->dimProb * sizeof (double)); 
	for (i = 0; i < dimStates_; i++) {
		if (probStates_ != 0 && probStates_ [i] == 0.0) {
	continue;
		}
		if (states_ != 0) {
			state = (void *) ((char *) states_ + i * sizeStates_);
		}
		for (size_t j = oldVec->minProb; j <= oldVec->maxProb; j++) {
			if ((probCond = oldVec->prob [j]) == 0.0) {
		continue;
			}
			(* oldVec->val) (oldVec->value, j, oldVec->indexParam);
			probCond *= probStates_ != 0 ? probStates_ [i] :
				(*condProbFct_) (state, oldVec->value, probParam_);
			if (probCond == 0.0) {
		continue;
			}
			(*updateFct_) (newVec->value, oldVec->value, state, updateParam_);
			newIndex = (*ind_) (newVec->value, indexParam_);
         if (newIndex < 0) {
Nks_Abort ("Nks_UpdateEither : newIndex < 0 (indexes must always be non-negative)");
         } else if (newVec->dimProb <= newIndex) {
				if (isProb_) {
					probNeglected += probCond;
				}
			} else {
				newVec->prob [newIndex] += probCond;
				minProb = MIN(minProb, newIndex);
				maxProb = MAX(maxProb, newIndex);
			}
		}
	}
	current = current == 0 ? 1 : 0;
	if (! isProb) {
		isProb = FALSE;
	}
	newVec->minProb = minProb;
	newVec->maxProb = maxProb;
	newVec->sizeValue = sizeValue_;
	newVec->val = val_;
	newVec->ind = ind_;
	newVec->indexParam = indexParam_;
MSG("Leaving Nks_DynProgProb::UpdateEither")
return TRUE;
}

/*****************************************************************************
 
************************** ALLOCATION ROUTINES *******************************
 
*****************************************************************************/

void Nks_DynProgProb::Initialize (   // intializes dynamic prog value
Nks_DynProgProb &initial,            // alias to initialization
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
void *initValue_,                    // initial value 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for conversion 
size_t dimProb_)                     // dim(prob) = max (# values) 
{
MSG("Entering Nks_DynProgProb::Initialize")
	if (dimProb_ == 0 || sizeValue_ == 0) {
Nks_Abort ("Nks_DynProgProb::Nks_DynProgProb : dimProb_ == 0 || sizeValue_ == 0");
	}
   initial.index = (Nks_VecIndex **) new Nks_VecIndex * [2];
   NKS_CATCH(initial.index [0] = new Nks_VecIndex (initValue_, ind_, indexParam_, maxSizeValue_, sizeValue_, val_, dimProb_), \
      std::bad_alloc, "Nks_DynProgProb::Nks_DynProgProb");
   NKS_CATCH(initial.index [1] = new Nks_VecIndex (maxSizeValue_, dimProb_),
      std::bad_alloc, "Nks_DynProgProb::Nks_DynProgProb");
	initial.maxSizeValue = maxSizeValue_;
	initial.current = 0;
	initial.probNeglected = 0.0;
	initial.isProb = TRUE;
MSG("Leaving Nks_DynProgProb::Initialize")
}

void Nks_DynProgProb::Destroy ()  // destroys dynamic prog value
{
MSG("Entering Nks_DynProgProb::Destroy")
   for (size_t i = 0; i < 2; i++) {
      delete index [i];
      index [i] = 0;
   }
   delete index;
   index = 0;
MSG("Leaving Nks_DynProgProb::Destroy")
}
