#ifndef NKSDYNPROGPROB_
#define NKSDYNPROGPROB_

#include <nks.h>
#include <nksglobal.h>
#include <nksmathdef.h>
#include <nksintmath.h>
#include <nksutil.h>
#include <nksio.h>

#ifdef Nks_UpdateFct
#undef Nks_UpdateFct
#endif
typedef void Nks_UpdateFct (void *newValue, void *oldValue, void *state, void *updateParam);

#ifdef Nks_IndexCondProbFct 
#undef Nks_IndexCondProbFct
#endif
typedef double Nks_IndexCondProbFct (void *newState, void *oldValue, void *param);

#ifdef Nks_IndexValueFct
#undef Nks_IndexValueFct
#endif
typedef long Nks_IndexValueFct  (void *value, void *param);

#ifdef Nks_ValueIndexFct
#undef Nks_ValueIndexFct
#endif
typedef void Nks_ValueIndexFct (void *value, long index, void *param);

#ifdef Nks_BoolFct
#undef Nks_BoolFct
#endif
typedef bool Nks_BoolFct (double prob, void *value);

#ifdef Nks_VecIndex
#undef Nks_VecIndex
#endif
class Nks_VecIndex {

   friend class Nks_DynProgProb;
   friend void Sss_ConvolveUpdate (void *newVecValue, void *oldVecValue, void *state, void *updateParam);

private:

Nks_VecIndex (
void *initValue_,                    // initial value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for conversion 
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
Nks_ValueIndexFct *val_,             // index -> value 
size_t dimProb_);                    // dim(prob) = max (# values) 

Nks_VecIndex (
void *initValue_,                    // initial value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for conversion 
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t dimProb_);                    // dim(prob) = max (# values) 

Nks_VecIndex (
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t dimProb_);                    // dim(prob) = max (# values) 

~Nks_VecIndex ();                    // parameters for conversion 

private:

	size_t dimProb;                   // dim(prob) = max (# values) 
	double *prob;                     // prob [0...dimProb-1] 
	long minProb;                     // min (non-0 index in prob) 
	long maxProb;                     // max (non-0 index in prob) 
	size_t sizeValue;                 // sizeof (value) 
	void *value;                      // workspace 
	Nks_ValueIndexFct *val;           // index -> value 
	Nks_IndexValueFct *ind;           // value -> index 
	void *indexParam;                 // parameters for conversion 

inline size_t ModIndex (long j) {return (size_t) Nks_Mod (j, (long) dimProb);};
inline double &ProbModIndex (long j) {return prob [(size_t) Nks_Mod (j, (long) dimProb)];};

Nks_VecIndex (                       // initialization of vector index 
const Nks_VecIndex &vecIndex_);      // vector index

Nks_VecIndex &operator= (            // assignment of vector index 
const Nks_VecIndex &vecIndex_);      // vector index

}; 
#ifdef VecIndex
#undef VecIndex
#endif
typedef Nks_VecIndex                 VecIndex;    

#ifdef Nks_DynProgProb
#undef Nks_DynProgProb
#endif
class	Nks_DynProgProb {

public:

Nks_DynProgProb (                    // inputs initial dynamic prog value 
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
void *initValue_,                    // initial value 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                  // parameters for conversion 
size_t dimProb_);                     // dim(prob) = max (# values) 

void Reinitialize (                  // reinitializes initial dynamic prog value 
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
void *initValue_,                    // initial value 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                  // parameters for conversion 
size_t dimProb_);                     // dim(prob) = max (# values) 

~Nks_DynProgProb ();

void DeleteCalculationSpace ();      // deletes unused workspace 

void NewCalculationSpace ();         // reallocates unused workspace 

void MoreCalculationSpace (          // extends dynamic prog values 
size_t maxSizeValue_,                // dim(prob) = new max (# values) 
size_t dimProb_);                    // expected maximum sizeof (value) 

bool Update (                        // updates dynamic prog probs 
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for index function 
size_t dimStates_,                   // #(states) 
size_t sizeStates_,                  // memory size for states 
void *states_,                       // void *to beginning of states array 
bool isProb_,                        // ? is probStates a vector of prob's ? 
double *probStates_);                // corresponding array of states prob's 

bool UpdateConditional (             // updates dynamic prog probs cond'ally 
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for index function 
size_t dimStates_,                   // #(states) 
size_t sizeStates_,                  // memory size for states 
void *states_,                       // void *to beginning of states array 
bool isProb_,                        // ? does condProbFct produce prob's ? 
Nks_IndexCondProbFct *condProbFct_,  // func producing conditional prob's 
void *probParam_);                   // parameters for conditional prob's 

bool Traverse (                      // outputs copies of dynamic prog probs 
Nks_BoolFct *boolfct_);              // BoolFct is applied to all values 

bool ConvolveVecIntoVec (            // convolves vecFrom into vecTo 
Nks_DynProgProb *dynProgProbFrom_,   // Nks_DynProgProb : pass as a void *
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index function for vector of values 
Nks_IndexValueFct *ind_,             // value function for indexes 
void *indexParam_);                  // parameters for index function 

inline size_t DimProb () // max (# values) 
   {return index [current]->dimProb;};

inline long MinProb () // min (non-0 index in prob) 
   {return index [current]->minProb;};

inline long MaxProb () // max (non-0 index in prob) 
   {return index [current]->maxProb;};

inline long DimActiveProb () // range of non-0 indexes in prob
   {return MaxProb () - MinProb () + 1;};

inline double  ProbNeglected () // probability mass neglected 
   {return probNeglected;};

inline bool IsProb () // ? is prob a probability ? 
   {return isProb;};

private:

	size_t maxSizeValue;              // expected maximum sizeof (value) 
	size_t current;                   // current index : [0] or [1] 
	double probNeglected;             // probability mass neglected 
	bool isProb;                      // TRUE while prob is a probability 
	Nks_VecIndex **index;             // index 

bool UpdateEither (                  // updates dynamic prog probs 
Nks_UpdateFct *updateFct_,           // updating function 
void *updateParam_,                  // parameters for update 
size_t sizeValue_,                   // memory size for newVec values 
Nks_ValueIndexFct *val_,             // index function for vector of values 
Nks_IndexValueFct *ind_,             // value function for indexes 
void *indexParam_,                   // parameters for index function 
size_t dimStates_,                   // #(states) 
size_t sizeStates_,                  // memory size for states 
void *states_,                       // void *to beginning of states array 
bool isProb_,                        // ? is probStates a vector of prob's ? 
double *probStates_,                 // corresponding array of states prob's 
Nks_IndexCondProbFct *condProbFct_,  // func producing conditional prob's 
void *probParam_);                   // parameters for conditional prob's 

void Initialize (                    // initializes initial dynamic prog value 
Nks_DynProgProb &initial,            // alias to initialization
size_t maxSizeValue_,                // expected maximum sizeof (value) 
size_t sizeValue_,                   // sizeof (value) 
void *initValue_,                    // initial value 
Nks_ValueIndexFct *val_,             // index -> value 
Nks_IndexValueFct *ind_,             // value -> index 
void *indexParam_,                   // parameters for conversion 
size_t dimProb_);                    // dim(prob) = max (# values) 

void Destroy ();                     // destroys dynamic prog value

Nks_DynProgProb (const Nks_DynProgProb &dynProgProb);

Nks_DynProgProb &operator= (         // assignment of dynamic prog probs 
const Nks_DynProgProb &dynProgProb_);// dynamic prog probs

};
#ifdef DynProgProb
#undef DynProgProb
#endif
typedef Nks_DynProgProb              DynProgProb;

/*****************************************************************************
 
******************** DYNAMIC PROGRAMMING SIGNIFICANCES ***********************
  
*****************************************************************************/

#endif //!NKSDYNPROGPROB_ 
