#include <nksio.h>

int Nks_AppendAndCloseFile (         // opens, appends, and closes file
const char *const nameOut,           // output file : status : "a"
const char *const format,            // format string
...)
{
   if (nameOut == 0 || ! strcmp (nameOut, "")) {
return NULL_FILE;
   }
	va_list args;
	FILE *file = fopen (nameOut, "a");
	if (file == NULL) {
return EOF;
	}
	va_start (args, format);
	int status = vfprintf (file, format, args); // error = EOF
	va_end (args);
	fclose (file);
return status;
}

int Nks_WriteAndCloseFile (          // opens, overwrites, and closes file
const char *const nameOut,           // output file : status : "a"
const char *const format,            // format string
...)
{
   if (nameOut == 0 || ! strcmp (nameOut, "")) {
return NULL_FILE;
   }
	va_list args;
	FILE *file = fopen (nameOut, "w");
	if (file == NULL) {
return EOF;
	}
	va_start (args, format);
	int status = vfprintf (file, format, args); // error = EOF
	va_end (args);
	fclose (file);
return status;
}

void Nks_AbortFormat (               // aborts with a formatted message
const char *const format,            // format string
...)
{
	va_list args;
	va_start (args, format);
	int status = vfprintf (stderr, format, args); // error = EOF
	va_end (args);
   (void) fprintf (stderr, "\n");
abort ();
}

void Nks_Abort (                     // aborts with a string
const char *const message)           // error message
{
	Nks_AbortFormat ("%s", message);
}

bool Nks_ReadLineThroughT (          // assumes buf can handle one line of input
std::ifstream &in,                   // input filestream
size_t dimBuf,                       // #(buf)
char *buf,                           // string before '!' from next line
char *t)                             // termination characters (not white space)
{
MSG("Entering Nks_ReadLinesThroughT")
   size_t i = 0;
   size_t j = 0;
   size_t count = 0; 
   char *pbuf = buf;
   char *pt = t;
   if (! in) Nks_Abort ("Nks_ReadLineThroughT : Input filestream is not valid.");
   while (in.getline (buf, dimBuf, '\n')) {
      if (dimBuf <= (count = (size_t) in.gcount ())) {
Nks_Abort ("Nks_ReadLineThroughT : Line is longer than buffer size.");
      }
      buf [count - 1] = '\0';
      bool allspace = true;
      for (pbuf = buf; *pbuf != '\0'; pbuf++) {
         if (isspace (*pbuf)) { // skip white space characters
         } else {               // first non-white space character
            for (pt = t; *pt != '\0'; pt++) {
               if (*pbuf == *pt) { // first non-white space is a terminator
                  if (allspace) {
   goto nextline;
                  } else {
                     *pbuf = '\0';
return true;
                  }
               }
            }
            allspace = false;
         }        
      }
nextline:;
   }
   buf [0] = '\0';
return false;                         // buf is empty and eof is reached 
MSG("Leaving Nks_ReadLinesThroughT")
}

void Nks_StringToReal (             // reads single real values from a string
char *str,                          // input : string
double *x)                          // output : real value 
{
	if (strcmp (str, "NaN") == 0) {
      *x = -HUGE_VAL;
   } else if (sscanf (str, "%lg", x) != 1) {
Nks_AbortFormat ("Nks_TikhPoint::ReadReal : %s does not contain a real number.", str);
   }
}


