#ifndef NKSIO_
#define NKSIO_

#include <nks.h>
#include <nksglobal.h>

const int NULL_FILE = -2;

int Nks_AppendAndCloseFile (         // opens, appends, and closes file
const char *const nameOut,           // output file : status : "a"
const char *const format,            // format string
...);

int Nks_WriteAndCloseFile (          // opens, overwrites, and closes file
const char *const nameOut,           // output file : status : "a"
const char *const format,            // format string
...);

inline int Nks_EraseFile (           // opens, erases, and closes file
const char *const nameOut)           // output file : status : "a"
{return Nks_WriteAndCloseFile (nameOut, "");};

void Nks_AbortFormat (               // aborts with a formatted message
const char *const format,            // format string
...);

void Nks_Abort (                     // aborts with a string
const char *const message);          // error message

// String-based routines

bool Nks_ReadLineThroughT (          // assumes buf can handle one line of input
std::ifstream &in,                   // input filestream
size_t dimBuf,                       // #(buf)
char *buf,                           // string before '!' from next line
char *t = "!");                      // termination characters (not white space)

void Nks_StringToReal (             // reads single real values from a string
char *str,                          // input : string
double *x);                         // output : real value 

#undef AppendAndCloseFile
#define AppendAndCloseFile	          Nks_AppendAndCloseFile

#undef WriteAndCloseFile
#define WriteAndCloseFile            Nks_WriteAndCloseFile

#undef EraseFile
#define EraseFile                    Nks_EraseFile

#undef AbortFormat
#define AbortFormat	                Nks_AbortFormat

#undef Abort
#define Abort                        Nks_Abort

#undef ReadLineThroughT
#define ReadLineThroughT             Nks_ReadLineThroughT

#undef StringToReal
#define StringToReal                 Nks_StringToReal

namespace Nks {

   static const size_t DIM_BUF = 1024;
   static const size_t dimBuf = 1024;
   static char buf [DIM_BUF];
   static int intbuf = 0;
   static long longbuf = 0L;
   static float floatbuf = 0.0F;
   static double doublebuf = 0.0;

   template <class T>
   void OutVector (
   const char *const outFile,       // output file name
   const char *const fmtName,       // format for vector name
   const char *const name,          // vector name
   const char *const fmt,           // format for each vector element
   const T *const vec,              // vector
   long m, long n)                  // m <= j < n                       
   {
      Nks_AppendAndCloseFile (outFile, fmtName, name);
      for (long j = m; j <= n; j++) {
         Nks_AppendAndCloseFile (outFile, fmt, (T) vec [j]);
      }
      Nks_AppendAndCloseFile (outFile, "\n");
   }

   template <class T>
   void OutMatrix (
   const char *const outFile,       // output file name
   const char *const fmtName,       // format for vector name
   const char *const name,          // vector name
   const char *const fmt,           // format for each vector element
   const T *const *const mat,       // matrix
   long m1, long n1,                // m1 <= j1 < n1 
   long m2, long n2,                // m2 <= j2 < n2  
   bool transpose = false)
   {
      Nks_AppendAndCloseFile (outFile, fmtName, name);
      if (transpose) {
         for (long j2 = m2; j2 <= n2; j2++) {
            for (long j1 = m1; j1 <= n1; j1++) {
               Nks_AppendAndCloseFile (outFile, fmt, (T) mat [j1][j2]);
            }
            Nks_AppendAndCloseFile (outFile, "\n");
         }
      } else {
         for (long j1 = m1; j1 <= n1; j1++) {
            for (long j2 = m2; j2 <= n2; j2++) {
               Nks_AppendAndCloseFile (outFile, fmt, (T) mat [j1][j2]);
            }
            Nks_AppendAndCloseFile (outFile, "\n");
         }
      }
   }

   template <class T>
   void OutTriMatrix (
   const char *const outFile,       // output file name
   const char *const fmtName,       // format for vector name
   const char *const name,          // vector name
   const char *const fmt,           // format for each vector element
   const T *const *const *const tri,// trimatrix
   long m1, long n1,                // m1 <= j1 < n1 
   long m2, long n2,                // m2 <= j2 < n2 
   long m3, long n3)                // m3 <= j3 < n3
   {
      Nks_AppendAndCloseFile (outFile, fmtName, name);
      for (long j1 = m1; j1 <= n1; j1++) {
         for (long j2 = m2; j2 <= n2; j2++) {
            for (long j3 = m3; j3 <= n3; j3++) {
               Nks_AppendAndCloseFile (outFile, fmt, (T) tri [j1][j2][j3]);
            }
            Nks_AppendAndCloseFile (outFile, "\n");
         }
         Nks_AppendAndCloseFile (outFile, "\n");
      }
   }

}

#endif //!NKSIO_
