#ifndef NKSMATHDEF_
#define NKSMATHDEF_

#include <nks.h>
#include <nksintmath.h>

// Constants

const double NKS_ROOT_2        =      1.4142135623730951e+000; // square root of 2.0
const double NKS_ROOT_10       =      3.1622776601683795e+000; // square root of 10.0

const double NKS_PI            =      3.1415926535897931e+000; // pi = ratio circumference to diameter
const double NKS_ROOT_PI       =      1.7724538509055159e+000; // square root of pi
const double NKS_ROOT_2_PI     =      2.5066282746310002e+000; // square root of 2.0 * pi

const double NKS_E             =      2.7182818284590451e+000; // base of natural logarithms
const double NKS_LN_2          =      6.9314718055994529e-001; // natural log of 2.0
const double NKS_LN_10         =      2.3025850929940459e+000; // natural log of 10.0
const double NKS_LN_LN_2       =     -3.6651292058166435e-001; // log of log of 2.0

const double NKS_EULER         =      0.5772156649015325e+000; // Euler's constant

const double ROOT_2            =      NKS_ROOT_2;              // square root of 2.0
const double ROOT_10           =      NKS_ROOT_10;             // square root of 10.0

const double PI                =      NKS_PI;                  // pi     = ratio circumference to diameter
const double ROOT_PI           =      NKS_ROOT_PI;             // square root of pi
const double ROOT_2_PI         =      NKS_ROOT_2_PI;           // square root of 2.0 * pi

const double E                 =      NKS_E;                   // base of natural logarithms
const double LN_2              =      NKS_LN_2;                // natural log of 2.0
const double LN_10             =      NKS_LN_10;               // natural log of 10.0
const double LN_LN_2           =      NKS_LN_LN_2;             // log of log of 2.0

const double EULER             =      NKS_EULER;               // Euler's constant

const bool   TRUE              =      1;                       // logical true
const bool   FALSE             =      0;                       // logical false

// Function Definitions

#ifndef NKS_EXP_2                   
#define NKS_EXP_2(A)                  (exp (NKS_LN_2 * (A)))
#endif
#undef EXP_2
#define EXP_2 NKS_EXP_2 
#ifndef NKS_LOG_2                    
#define NKS_LOG_2(A)                  (log (A) / NKS_LN_2)
#endif
#undef LOG_2
#define LOG_2 NKS_LOG_2 
#ifndef NKS_EXP_10
#define NKS_EXP_10(A)                 (exp (NKS_LN_10 * (A)))
#endif
#undef EXP_10
#define EXP_10 NKS_EXP_10 
#ifndef NKS_LOG_10
#define NKS_LOG_10(A)                 (log (A) / NKS_LN_10)
#endif
#undef LOG_10
#define LOG_10 NKS_LOG_10 
#ifndef NKS_BITStoNATS
#define NKS_BITStoNATS(x)             ((x) * NKS_LN_2)
#endif
#undef BITStoNATS
#define BITStoNATS NKS_BITStoNATS 
#ifndef NKS_NATStoBITS
#define NKS_NATStoBITS(x)             ((x) / NKS_LN_2)
#endif
#undef NATStoBITS
#define NATStoBITS NKS_NATStoBITS 

#ifndef NKS_ABS
#define NKS_ABS(x)                    ((x) >= 0.0 ? (x) : -(x))
#endif
#undef ABS
#define ABS NKS_ABS 
#ifndef NKS_MAX
#define NKS_MAX(x,y)                  ((x) >= (y) ? (x) : (y))
#endif
#undef MAX
#define MAX NKS_MAX 
#ifndef NKS_MIN
#define NKS_MIN(x,y)                  ((x) <= (y) ? (x) : (y))
#endif
#undef MIN
#define MIN NKS_MIN 
#ifndef NKS_PLUS
#define NKS_PLUS(x)                   NKS_MAX((x), 0)
#endif
#undef PLUS
#define PLUS NKS_PLUS 
#ifndef NKS_MINUS
#define NKS_MINUS(x)                  NKS_MAX(-(x), 0)
#endif
#undef MINUS
#define MINUS NKS_MINUS 
#ifndef NKS_SQRT
#define NKS_SQRT(x)                   sqrt (NKS_PLUS(x))
#endif
#undef SQRT
#define SQRT NKS_SQRT 
inline float Nks_Square (float x) {return x * x;};
inline double Nks_Square (double x) {return x * x;};
#undef Square
#define Square Nks_Square 

#ifndef NKS_BOUND
#define NKS_BOUND(x,xlo,xhi)          ((x)<(xlo)?(xlo):((x)>(xhi)?(xhi):(x)))   
#endif
#undef BOUND
#define BOUND NKS_BOUND 
#ifndef NKS_PROBABILITY
#define NKS_PROBABILITY(x)            NKS_BOUND(x, 0.0, 1.0)
#endif
#undef PROBABILITY
#define PROBABILITY NKS_PROBABILITY 
#ifndef NKS_ERROR_PROBABILITY
#define NKS_ERROR_PROBABILITY         -HUGE_VAL
#endif
#undef ERROR_PROBABILITY
#define ERROR_PROBABILITY NKS_ERROR_PROBABILITY

#ifndef NKS_NEAREST_INTEGER
#define NKS_NEAREST_INTEGER(x)        floor ((x) + 0.5L)
#endif
#undef NEAREST_INTEGER
#define NEAREST_INTEGER NKS_NEAREST_INTEGER 
#ifndef NKS_MINUS_ONE_POWER
#define NKS_MINUS_ONE_POWER(x)        (Nks_Mod (x, 2) ? -1 : 1)
#endif
#undef MINUS_ONE_POWER
#define MINUS_ONE_POWER NKS_MINUS_ONE_POWER 

inline short Nks_BoolToInteger (bool x) {return x ? 1 : 0;};
#undef BoolToInteger
#define BoolToInteger Nks_BoolToInteger 
inline bool Nks_IntegerToBool (long x) {return x == 0L ? false : true;};
#undef NIntegerToBool
#define NIntegerToBool Nks_IntegerToBool 
inline double Nks_KroneckerDelta (long i, long j) {return i == j ? 1.0 : 0.0;}; 
#undef KroneckerDelta
#define KroneckerDelta Nks_KroneckerDelta 

inline double Nks_Signum (double x) {return x > 0.0 ? 1.0 : (x < 0.0 ? -1.0 : 0.0);}; // Signum Function
#undef Signum
#define Signum Nks_Signum 
inline double Nks_Heaviside (double x) {return x >= 0.0 ? 1.0 : 0.0;}; // Heaviside Function
#undef Heaviside
#define Heaviside Nks_Heaviside 
inline double Nks_Sign (double x, double y) {return (y >= 0.0 ? 1.0 : -1.0) * NKS_ABS (x);}; // Heaviside Function

// Approximation

inline bool Nks_Approx (double x, double y, double eps) {return (bool) (NKS_ABS(x - y) < NKS_ABS(eps));};
#undef Approx
#define Approx Nks_Approx 
inline bool Nks_RelApprox (double x, double y, double eps) {return Nks_Approx (x, y, eps * y);};
#undef RelApprox
#define RelApprox Nks_RelApprox 
inline bool Nks_AbsRelApprox (double x, double y, double tol, double rtol) {return Nks_Approx (x, y, tol) || Nks_RelApprox (x, y, rtol);};
#undef AbsRelApprox
#define AbsRelApprox Nks_AbsRelApprox 

// Float Rounding

const float NKS_FLT_THRESHOLD =       10.0F; // Rounding threshold
#undef FLT_THRESHOLD
#define FLT_THRESHOLD NKS_FLT_THRESHOLD 
const float NKS_FLT_ROUND  =         (NKS_FLT_THRESHOLD * FLT_EPSILON); // Rounding error threshold
#undef FLT_ROUND
#define FLT_ROUND NKS_FLT_ROUND 

// Double Rounding

const float NKS_DBL_THRESHOLD =       100.0; // Rounding threshold
#undef DBL_THRESHOLD
#define DBL_THRESHOLD NKS_DBL_THRESHOLD 
const double NKS_DBL_ROUND  =         (NKS_DBL_THRESHOLD * DBL_EPSILON); // Rounding error threshold
#undef DBL_ROUND
#define DBL_ROUND NKS_DBL_ROUND 

// Rounding Functions

inline bool Nks_Eq (float x, float y) {return Nks_RelApprox (x, y, NKS_FLT_ROUND);};
inline bool Nks_Eq (double x, double y) {return Nks_RelApprox (x, y, NKS_DBL_ROUND);};
#undef Eq
#define Eq Nks_Eq 
inline bool Nks_Ne (float x, float y) {return ! Nks_Eq (x, y);};
inline bool Nks_Ne (double x, double y) {return ! Nks_Eq (x, y);};
#undef Ne
#define Ne Nks_Ne 
inline bool Nks_Ge (float x, float y) {return (bool) (x - y >= -NKS_ABS(y) * NKS_FLT_ROUND);};
inline bool Nks_Ge (double x, double y) {return (bool) (x - y >= -NKS_ABS(y) * NKS_DBL_ROUND);};
#undef Ge
#define Ge Nks_Ge 
inline bool Nks_Gt (float x, float y) {return (bool) (x - y >   NKS_ABS(y) * NKS_FLT_ROUND);};
inline bool Nks_Gt (double x, double y) {return (bool) (x - y >   NKS_ABS(y) * NKS_DBL_ROUND);};
#undef Gt
#define Gt Nks_Gt 
inline bool Nks_Le (float x, float y) {return (bool) (x - y <=  NKS_ABS(y) * NKS_FLT_ROUND);};
inline bool Nks_Le (double x, double y) {return (bool) (x - y <=  NKS_ABS(y) * NKS_DBL_ROUND);};
#undef Le
#define Le Nks_Le 
inline bool Nks_Lt (float x, float y) {return (bool) (x - y <  -NKS_ABS(y) * NKS_FLT_ROUND);};
inline bool Nks_Lt (double x, double y) {return (bool) (x - y <  -NKS_ABS(y) * NKS_DBL_ROUND);};
#undef Lt
#define Lt Nks_Lt 

#endif //!NKSMATHDEF_
