#ifndef NKSPROBDIST_
#define NKSPROBDIST_

#include <nks.h>
#include <nksglobal.h>
#include <nksmathdef.h>
#include <nksio.h>
#include <nksutil.h>

/*****************************************************************************

************************* PROBABILITY DISTRIBUTIONS **************************
  
*****************************************************************************/

#ifdef Nks_LongMeasure
#undef Nks_LongMeasure
#endif
typedef double Nks_LongMeasure (long i, void *param);

#ifdef Nks_FiniteIntProbDist
#undef Nks_FiniteIntProbDist
#endif
class Nks_FiniteIntProbDist {

   friend double MeanSquareStar (double x, void *param);
   friend double MeanStar (double x, void *param);
   friend double ProbStar (double x, void *param);

public:

Nks_FiniteIntProbDist (              // contains parameters of a finite span probability distribution 
size_t dimProb = 0,                  // #(distinct values)          
long *values = 0,                    // values 
double *prob = 0);                   // probability of corresponding value  

Nks_FiniteIntProbDist ( // contains parameters of an integer probability distribution  
Nks_LongMeasure *longMeasure_,       // function returning a probability measure on the integers          
void *param_,                        // parameters for longMeasure_
long lower_,                         // lower bound of explicitly calculated values
long upper_);                        // upper bound of explicitly calculated values

Nks_FiniteIntProbDist (              // contains parameters of a finite span probability distribution 
const Nks_FiniteIntProbDist &step_); // finite span probability distribution

Nks_FiniteIntProbDist &operator= (// contains parameters of a finite span probability distribution 
const Nks_FiniteIntProbDist &finiteIntProbDist_); // finite span probability distribution

~Nks_FiniteIntProbDist ();

inline size_t DimProb () {return dimProb;} // #(distinct values)
inline long Value (size_t i) {return value [i];}; // i-th element of value
inline double Prob (size_t i) {return prob [i];}; // i-th element of prob
inline double Mu () {return mu;}; // step mean
inline double Sigma () {return sigma;}; // step standard deviation

double CalcMu ();                    // calculates mean  
double CalcSigma ();                 // calculates standard deviation 

protected:

   size_t dimProb;                   // #(distinct values)          
   long *value;                      // values 
   double *prob;                     // probability of corresponding value  
   double mu;                        // step mean  
   double sigma;                     // step standard deviation  

private:

void Nks_Initialize ( // contains parameters of a finite span probability distribution 
Nks_FiniteIntProbDist &init,             // alias to initialization
const Nks_FiniteIntProbDist &finiteIntProbDist_); // finite span probability distribution

};
#ifdef FiniteIntProbDist
#undef FiniteIntProbDist
#endif
typedef Nks_FiniteIntProbDist        FiniteIntProbDist;

const long LOWER_DUMMY = LONG_MAX;
const long UPPER_DUMMY = LONG_MIN;

#ifdef Nks_InfiniteIntProbDist
#undef Nks_InfiniteIntProbDist
#endif
class Nks_InfiniteIntProbDist :      // contains parameters of an integer probability distribution  
protected Nks_FiniteIntProbDist {    // derived class 

public:

Nks_InfiniteIntProbDist (// contains parameters of an integer probability distribution  
Nks_LongMeasure *longMeasure_ = 0,   // function returning a probability measure on the integers          
void *param_ = 0,                    // parameters for longMeasure_
long lower_ = 0,                     // lower bound of explicitly calculated values
long upper_ = 0);                    // upper bound of explicitly calculated values

Nks_InfiniteIntProbDist ( // contains parameters of an integer probability distribution 
const Nks_InfiniteIntProbDist &infiniteIntProbDist_);

Nks_InfiniteIntProbDist &operator= (// contains parameters of a finite span probability distribution 
const Nks_InfiniteIntProbDist &infiniteIntProbDist_); // finite span probability distribution

~Nks_InfiniteIntProbDist ();

Nks_LongMeasure *LongMeasure () {return longMeasure;};
inline void *Param () {return param;};
inline long Lower () {return lower;};
inline long Upper () {return upper;};
inline long ProbabilitySurplus () {return probabilitySurplus;};
void ChangeLowerUpper (long lower_, long upper_);
inline void ChangeLower (long lower_) {ChangeLowerUpper (lower_, upper);};
inline void ChangeUpper (long upper_) {ChangeLowerUpper (lower, upper_);};

protected:

   Nks_LongMeasure *longMeasure;     // function returning a probability measure on the integers         
   void *param;                      // parameters for longMeasure
   long lower;                       // lower bound of explicitly calculated values
   long upper;                       // upper bound of explicitly calculated values
   double probabilitySurplus;        // extra probability outside of range

private:

void Nks_Initialize ( // contains parameters of a finite span probability distribution 
Nks_InfiniteIntProbDist &init,             // alias to initialization
const Nks_InfiniteIntProbDist &infiniteIntProbDist_); // finite span probability distribution

};
#ifdef InfiniteIntProbDist
#undef InfiniteIntProbDist
#endif
typedef Nks_InfiniteIntProbDist        InfiniteIntProbDist;

#endif //!NKSPROBDIST_ 
