#include <nksrandomwalk.h>

/*static*/ double ProbStar (double x, void *param)
{
MSG("Entering ProbStar")
   Nks_FiniteIntProbDist *p = (Nks_FiniteIntProbDist *) param;
   double sum = 0.0;
   for (size_t i = 0; i < p->dimProb; i++) {
      sum += p->prob [i] * exp (x * (double) p->value [i]);
   }
MSG("Leaving ProbStar")
return sum;
}

/*static*/ double MeanStar (double x, void *param)
{
MSG("Entering MeanStar")
   Nks_FiniteIntProbDist *p = (Nks_FiniteIntProbDist *) param;
   double sum = 0.0;
   for (size_t i = 0; i < p->dimProb; i++) {
      sum += (double) p->value [i] * 
         p->prob [i] * exp (x * (double) p->value [i]);
   }
MSG("Leaving MeanStar")
return sum;
}

/*static*/ double MeanSquareStar (double x, void *param)
{
MSG("Entering MeanSquareStar")
   Nks_FiniteIntProbDist *p = (Nks_FiniteIntProbDist *) param;
   double sum = 0.0;
   for (size_t i = 0; i < p->dimProb; i++) {
      sum += (double) p->value [i] * (double) p->value [i] * 
         p->prob [i] * exp (x * (double) p->value [i]);
   }
MSG("Leaving MeanSquareStar")
return sum;
}

static void valConvolve (void *value, long index, void *param)
{
MSG("Entering valConvolve")
   long *score = (long *) value;
   long *minValue = (long *) param;
   *score = index + *minValue;
MSG("Leaving valConvolve")
}

static long indConvolve (void *value, void *param)
{
MSG("Entering indConvolve")
   long *score = (long *) value;
   long *minValue = (long *) param;
MSG("Leaving indConvolve")
return *score - *minValue;
}

static void updateConvolve (
void *newValue,
void *oldValue,
void *value,
void *funcParam)
{
MSG("Entering updateConvolve")
	long *newScore = (long *) newValue;
	long *oldScore = (long *) oldValue;
	long *score = (long *) value;
	*newScore = *oldScore + *score;
MSG("Leaving updateConvolve")
}

static double setP_term = 0.0;
static double setQ_term = 0.0;
static double setLambda = 0.0;
static size_t setI = 0;

static bool AddSums (double prob, void *value)
{
MSG("Entering AddSums")
   long *score = (long *) value;
   if (0 <= *score) {
      setP_term += prob;
   } else {
      setQ_term += prob * exp (setLambda * (double) *score);
   }
MSG("Leaving AddSums")
return TRUE;
}

Nks_FiniteIntRandomWalk::Nks_FiniteIntRandomWalk (// contains random walk parameters 
size_t dimProb_,                     // #(distinct values)          
long *value_,                        // values 
double *prob_)                       // probability of corresponding value  
{
MSG("Entering Nks_FiniteIntRandomWalk::Nks_FiniteIntRandomWalk 3")
   NKS_CATCH(step = new Nks_FiniteIntProbDist (dimProb_, value_, prob_),
      std::bad_alloc, "Nks_FiniteIntRandomWalk::Nks_FiniteIntRandomWalk");
   if (0.0 <= step->CalcMu ()) {     // random walk does not have negative drift
      lambda = 0.0;                  // lambda for associated random walk
      lambda0 = 0.0;                 // lambda for minimum expectation (exp (lambda * score))
      rho = 0.0;                     // minimum expectation (exp (lambda * score))
      k_minus = 0.0;                 // K_minus for random walk : exponential prefactor
      k_plus = 0.0;                  // K_plus for random walk : exponential prefactor
      meanLength = 0.0;              // expected renewal length
      meanSquareLength = 0.0;        // expected renewal length
      muStar = 0.0;                  // step mean for associated random walk (relative entropy)
      sigmaStar = 0.0;               // step standard deviation for associated random walk
return;
   }
// Calculating lambda and related quantities
   const double REL_TOL = 1.0E-10;
   double p = -log (prob_ [dimProb_ - 1]) / (double) value_ [dimProb_ - 1];
   double q = 0.5 * p;
   while (1.0 <= ProbStar (q, (void *) step)) {
      p = q;
      q *= 0.5;
   }
   double x = 0.5 * (p + q);
   lambda = Nks_RootFinder (1.0, ProbStar, MeanStar, (void *) step, p, x, q, REL_TOL * NKS_ABS(p - q));
   muStar = MeanStar (lambda, (void *) step);
   sigmaStar = NKS_SQRT (MeanSquareStar (lambda, (void *) step) - muStar * muStar);
   x = 0.5 * lambda;
   lambda0 = Nks_RootFinder (0.0, MeanStar, MeanSquareStar, (void *) step, 0.0, x, lambda, REL_TOL * NKS_ABS(p - q));
   rho = ProbStar (lambda0, (void *) step);
// Calculating k
   size_t n = (size_t) (log (REL_TOL * (1.0 - rho)) / log (rho));
   long minValue = n * value_ [0];
   long initValue = 0;
//
// Start dynamic programming probability calculation using notation in
//
// Mott R. and Tribe R. (1999)
// J. Computational Biology 6(1):91-112
//
// This program follows Mott and Tribe in using departure into (-Inf, 0)
// rather than (-Inf, 0].
//
// The formula for meanSquareLength is a modification of the generating
// function in 
//
// Karlin S. and Taylor H.M.(1981)
// A Second Course in Stochastic Processes, p. 480
//
// Note there is an error in Eq (6.19) there, which is corrected in Eq (6.20)
//
   double P_straightSum = 0.0;
   double P_sum = 0.0;
   double Q_sum = 0.0;
   Nks_DynProgProb dynProgProb (sizeof (long), sizeof (long), 
      &initValue, valConvolve, indConvolve, &minValue, 
      n * (value_ [dimProb_ - 1] - value_ [0] + 1));
// Initialize static variable
   setLambda = lambda;
// Updating loop for dynamic programming
   for (setI = 1; setI <= n; setI++) {
		(void) dynProgProb.Update (updateConvolve, 0,
			sizeof (long), valConvolve, indConvolve, &minValue,
			dimProb_, sizeof (long), (void *) value_, TRUE, prob_);
// Initialize static variables
      setP_term = setQ_term = 0.0;            
// Update sums
      (void) dynProgProb.Traverse (AddSums);
      P_straightSum += setP_term; 
      P_sum += setP_term / (double) setI;
      Q_sum += setQ_term / (double) setI;
   }
// Calculation of fluctuation sum quantities
   double P_expsum = exp (-P_sum);
   double Q_expsum = exp (-Q_sum);
   double s = 1.0 / (exp (lambda) - 1.0) * P_expsum * Q_expsum / muStar;
   double r = s * Q_expsum;
   k_minus = r * P_expsum;
   k_plus = k_minus * exp (lambda);
   meanLength = 1.0 / P_expsum;
   meanSquareLength = (1.0 + 2.0 * P_straightSum) * meanLength;
MSG("Leaving Nks_FiniteIntRandomWalk::Nks_FiniteIntRandomWalk 3")
}

Nks_FiniteIntRandomWalk::~Nks_FiniteIntRandomWalk ()
{
MSG("Entering Nks_FiniteIntRandomWalk::~Nks_FiniteIntRandomWalk")
   delete step; 
   step = 0;
MSG("Leaving Nks_FiniteIntRandomWalk::~Nks_FiniteIntRandomWalk")
}
