#ifndef NKSRANDOMWALK_
#define NKSRANDOMWALK_
#include <nks.h>
#include <nksglobal.h>
#include <nksutil.h>
#include <nksrootfinder.h>
#include <nksmathdef.h>
#include <nksio.h>
#include <nksdynprogprob.h>
#include <nksprobdist.h>

/*****************************************************************************
 
*************************** RANDOM WALK PARAMETERS ***************************
  
*****************************************************************************/

#ifdef Nks_FiniteIntRandomWalk
#undef Nks_FiniteIntRandomWalk
#endif
class	Nks_FiniteIntRandomWalk {      // contains random walk parameters

public:

Nks_FiniteIntRandomWalk (            // contains random walk parameters 
size_t dimProb,                      // #(distinct values)          
long *values,                        // values 
double *prob);                       // probability of corresponding value  

~Nks_FiniteIntRandomWalk ();

double Lambda () {return lambda;};   // lambda for associated random walk
double Lambda0 () {return lambda0;}; // lambda for minimum expectation (exp (lambda * score))
double Rho () {return rho;};         // minimum expectation (exp (lambda * score))
double K_minus () {return k_minus;}; // K_minus for random walk : exponential prefactor
double K_plus () {return k_plus;};   // K_plus for random walk : exponential prefactor
double MeanLength () {return meanLength;};   // expected renewal length
double MeanSquareLength () {return meanSquareLength;};   // expected renewal length squared
double MuStar () {return muStar;};   // step mean for associated random walk (relative entropy)
double SigmaStar () {return sigmaStar;};   // step standard deviation for associated random walk

private:

   Nks_FiniteIntProbDist *step;      // step distribution (span assumed to be 1)
   double lambda;                    // lambda for associated random walk
   double lambda0;                   // lambda for minimum expectation (exp (lambda * score))
   double rho;                       // minimum expectation (exp (lambda * score))
   double k_minus;                   // K_minus for random walk : exponential prefactor
   double k_plus;                    // K_plus for random walk : exponential prefactor
   double meanLength;                // expected renewal length
   double meanSquareLength;          // expected renewal length squared
   double muStar;                    // step mean for associated random walk (relative entropy)
   double sigmaStar;                 // step standard deviation for associated random walk

Nks_FiniteIntRandomWalk (              // initialization of random walk parameters 
const Nks_FiniteIntRandomWalk &finiteIntRandomWalk); // random walk parameters

Nks_FiniteIntRandomWalk &operator= (   // assignment of random walk parameters 
const Nks_FiniteIntRandomWalk &finiteIntRandomWalk); // random walk parameters

};
#ifdef FiniteIntRandomWalk
#undef FiniteIntRandomWalk
#endif
typedef Nks_FiniteIntRandomWalk        FiniteIntRandomWalk;

#endif //!NKSRANDOMWALK_ 
