
#ifndef NKSUTIL_
#define NKSUTIL_

#include <stdlib.h>
#include <string.h>
#include <memory>
#include <nksmathdef.h>
#include <nksio.h>

/*****************************************************************************
 
******************************* UTILITY ROUTINES  *****************************

*****************************************************************************/

void *Nks_MemNew (size_t size);

void *Nks_MemMore (void *p, size_t size);

void Nks_MemCopy (void *to, const void *from, size_t size);

void *Nks_MemFree (void *p);

void *Nks_MemSet (void *p, char c, size_t size);

void *Nks_MemZero (void *p, size_t size);

void Nks_ModularMemCopy (                          // circular copying routine
void *to,                                          // #(copied *to)
size_t toSize,                                     // #(size of *to)
size_t toMinIndex,                                 // #(placement of first copied element in *to)
const void *from,                                  // #(copied *from)
size_t fromSize,                                   // #(size of *from)
size_t fromMinIndex,                               // #(placement of first copied element in *from)
size_t size,                                       // #(elements to be copied)
size_t sizeSpace);                                 // #(size of elements to be copied)

void **Nks_NewMatrix (long iMin, long iMax, long jMin, long jMax, size_t sizeElement);

void Nks_DeleteMatrix (void **matrix, long iMin, long iMax, long jMin, long jMax, size_t sizeElement);

long **Nks_NewLongMatrix (long iMin, long iMax, long jMin, long jMax);

void Nks_DeleteLongMatrix (long **matrix, long iMin, long iMax, long jMin, long jMax);

namespace Nks {

   inline void Delete (void *&p) {delete p; p = 0;}

   inline void DeleteArray (void *&p) {delete [] p; p = 0;}

   void *NewVector (long iMin, long iMax, size_t sizeElement);

   inline void DeleteVector (void *&vector, long iMin, long iMax, size_t sizeElement)
   {
      delete [] ((char *) vector + sizeElement * iMin);
      vector = 0;
   }

   inline float *NewFloatVector (long iMin, long iMax) 
   {return (float *) NewVector (iMin, iMax, sizeof (float));}

   inline void DeleteFloatVector (float *&vector, long iMin, long iMax) 
   {DeleteVector ((void *&) vector, iMin, iMax, sizeof (float));}

   inline double *NewDoubleVector (long iMin, long iMax) 
   {return (double *) NewVector (iMin, iMax, sizeof (double));}

   inline void DeleteDoubleVector (double *&vector, long iMin, long iMax) 
   {DeleteVector ((void *&) vector, iMin, iMax, sizeof (double));}

   inline void **NewMatrix (long iMin, long iMax, long jMin, long jMax, size_t sizeElement)
   {return Nks_NewMatrix (iMin, iMax, jMin, jMax, sizeElement);}

   inline void DeleteMatrix (void **&matrix, long iMin, long iMax, long jMin, long jMax, size_t sizeElement)
   {
      Nks_DeleteMatrix (matrix, iMin, iMax, jMin, jMax, sizeElement);
      matrix = 0;
   }

   inline long **NewLongMatrix (long iMin, long iMax, long jMin, long jMax)
   {return (long **) NewMatrix (iMin, iMax, jMin, jMax, sizeof (long));}

   inline void DeleteLongMatrix (long **&matrix, long iMin, long iMax, long jMin, long jMax)
   {DeleteMatrix ((void **&) matrix, iMin, iMax, jMin, jMax, sizeof (long));}

   inline double **NewDoubleMatrix (long iMin, long iMax, long jMin, long jMax) 
   {return (double **) NewMatrix (iMin, iMax, jMin, jMax, sizeof (double));}

   inline void DeleteDoubleMatrix (double **&matrix, long iMin, long iMax, long jMin, long jMax) 
   {DeleteMatrix ((void **&) matrix, iMin, iMax, jMin, jMax, sizeof (double));}

   inline float **NewFloatMatrix (long iMin, long iMax, long jMin, long jMax) 
   {return (float **) NewMatrix (iMin, iMax, jMin, jMax, sizeof (float));}

   inline void DeleteFloatMatrix (float **&matrix, long iMin, long iMax, long jMin, long jMax) 
   {DeleteMatrix ((void **&) matrix, iMin, iMax, jMin, jMax, sizeof (float));}

}

#undef  MemNew
#define MemNew                    Nks_MemNew                 

#undef  MemMore
#define MemMore                   Nks_MemMore                 

#undef  MemCopy
#define MemCopy                   Nks_MemCopy                 

#undef  MemFree
#define MemFree                   Nks_MemFree

#undef  MemSet 
#define MemSet                    Nks_MemSet

#undef  MemZero
#define MemZero                   Nks_MemZero

#undef  ModularMemCopy
#define ModularMemCopy            Nks_ModularMemCopy

#undef  NewMatrix
#define NewMatrix                 Nks_NewMatrix

#undef  DeleteMatrix
#define DeleteMatrix              Nks_DeleteMatrix

#undef  NewLongMatrix
#define NewLongMatrix             Nks_NewLongMatrix

#undef  DeleteLongMatrix
#define DeleteLongMatrix          Nks_DeleteLongMatrix

#endif // NKSUTIL_
