#include<sstream>
#include "MCFgen.hpp"  // die
#include "glam.hpp"

void glam::parse_glam(const string & text, vector<alignment> & aln, const vector<uint> & seqlens)
{
  const string best_text = "Best alignment found:";
  const string width_text = "Width: ";  // the space is a kludge

  istringstream strm(text);
  string line;
  enum { WANT_BEST, WANT_WIDTH, WANT_POS } state = WANT_BEST;
  alignment a;
  uint seq = 0;

  while (getline(strm, line)) {
    if (state == WANT_BEST) {
      if (line.substr(0, best_text.size()) == best_text)
	state = WANT_WIDTH;
    } else if (state == WANT_WIDTH) {
      string::size_type idx = line.find(width_text);
      if (idx != string::npos &&
	  istringstream(line.substr(idx + width_text.size() - 1)) >> a.width)
	state = WANT_POS;
    } else {  // state == WANT_POS
      string s;
      uint start;
      char strand;
      // temporary istringstream >> string isn't allowed in C++!
      istringstream bogus1(line);
      istringstream bogus2(line);
      if (bogus1 >> s >> s && s == "absent")
	a.starts.push_back(0);
      else if (bogus2 >> s >> start >> s >> s >> strand)
	if (strand == '+') {
	  if (start + a.width > seqlens[seq] + 1)
	    mcf::die("Error: previous alignment exceeds sequence bounds");
	  a.starts.push_back(start);
	} else if (strand == '-') {
	  if (start > seqlens[seq] || start < a.width)
	    mcf::die("Error: previous alignment exceeds sequence bounds");
	  a.starts.push_back(2 * seqlens[seq] + 2 - start);
	} else
	  mcf::die("Error: missing +/- strand information in previous alignments file");
      else {  // there must be at least 1 line after the last alignment!
	if (a.starts.size() != seqlens.size())
	  mcf::die("Error: previous alignment has wrong number of sequences");
	aln.push_back(a);
	reflect_alignment(a, seqlens);
	aln.push_back(a);  // get the reverse-complement alignment too
	a.starts.clear();
	seq = 0;
	state = WANT_BEST;
	continue;  // avoid ++seq
      }
      ++seq;
    }
  }
}
